/*
 * The MIT License
 *
 * Copyright 2012 Massive Dynamics.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package biz.massivedynamics.modger.message;

import biz.massivedynamics.modger.Logger;

/**
 * A class that defines a single log message, which is essentially a string and message type
 * 
 * @author Cruz Bishop
 * @since 1.0.0.0
 */
public class Message implements Cloneable {
    
    /**
     * Creates a new log message
     * 
     * @param logger The logger to use
     * @param message The string to use
     * @since 1.0.0.0
     */
    public Message(Logger logger, String message) {
        this(logger, message, MessageType.REGULAR);
    }
    
    /**
     * Creates a new log message
     * 
     * @param logger The logger to use
     * @param message The string to use
     * @param type The message type to use
     * @since 1.0.0.0
     */
    public Message(Logger logger, String message, MessageType type) {
        this.logger = logger;
        this.message = message;
        this.type = type;
    }
    
    /**
     * The logger being used
     * 
     * @since 1.0.0.0
     */
    private Logger logger;
    
    /**
     * Gets the logger being used
     * 
     * @return The logger
     * @since 1.0.0.0
     */
    public Logger getLogger() {
        return this.logger;
    }
    
    /**
     * The message that this LogMessage holds
     * 
     * @since 1.0.0.0
     */
    private String message;
    
    /**
     * Gets the message
     * 
     * @return The message
     * @since 1.0.0.0
     */
    public String getMessage() {
        return this.message;
    }
    
    /**
     * Sets the message
     * 
     * @param message The message to use
     * @since 1.0.0.0
     */
    public void setMessage(String message) {
        this.message = message;
    }
    
    /**
     * The type of message this is
     * 
     * @since 1.0.0.0
     */
    private MessageType type;
    
    /**
     * Gets this message's type
     * 
     * @return The type
     * @since 1.0.0.0
     */
    public MessageType getType() {
        return this.type;
    }
    
    /**
     * Sets the message type
     * 
     * @param type The type to use
     * @since 1.0.0.0
     */
    public void setType(MessageType type) {
        this.type = type;
    }
    
    /**
     * Clones this Message
     * 
     * @return The cloned message
     * @since 1.0.0.0
     */
    @Override
    public Message clone() {
        return new Message(this.getLogger(), this.getMessage(), this.getType());
    }
    
    /**
     * Gets the length of the message
     * 
     * @return The length
     * @since 1.0.0.0
     */
    public int getLength() {
        return this.getMessage().length();
    }

    
}
